UNIT mTBruch;
// nderungen:
// 1.
// Function "ggT" benutzt Betrge, so dass auch Brche mit
// negativem Zhler oder Nenner richtig gekrzt werden.
// Hat eines der beiden Argumente den Wert 0, so ist das andere
// der ggT. Sonst gegenber SuM-Version unverndert.
// 2.
// Alle Referenzen auf SuM entfernt
// => "ganzeZahl" -> "Integer"; "Zahl" -> "Real"
// "zNenner", "zZaehler" sind private!
// 3.
// Die Unit "TBruch" produziert nun autonomere, "intelligentere" Brche:
// Die Setzung "Nenner = 0" wird ignoriert.
// Nach jeder nderung von Zhler oder Nenner wird der Bruch in die
// "Normalform" (Zhler aus Z, Nenner aus N) berfhrt ("normiere")
// und danach gekrzt. "kuerze(Voll)" kann dadurch "private" werden.
// Die Neucodierung von "TBruchrechner" ohne nderung der Ausgangsbrche
// macht die "technischen" Methoden "erweitereUm", "kuerzeUm" und "Kehrbruch"
// berflssig.
// Neu notwendig wird "setzeBruch", das nach einer kompletten nderung
// eines Bruches (ErgebnisBruch!) verhindert, dass Zhler oder Nenner,
// wer immer als erster Wert verndert wird, noch mit dem zu ber-
// schreibenden anderen Wert des alten Bruches verrechnet wird.
// 4.
// keine nderung

interface

// uses;

type
   TBruch = CLASS

     private

     // Attribute

        zNenner  : Integer;
        zZaehler : Integer;

     // Methoden

        function  ggt(p1,p2:Integer) : Integer;
        procedure normiere;
        procedure kuerze;

     public
        constructor init;

        // Prozeduren fr den Import von Daten
        procedure setzeBruch  (pZaehler,pNenner:Integer);
        procedure setzeZaehler(pZaehler:Integer);
        procedure setzeNenner (pNenner :Integer);

        // Prozeduren fr den Export von Daten
        function dezimalwert : Real;
        function nenner      : Integer;
        function zaehler     : Integer;

        destructor gibFrei;

   end;

implementation

//-------- ggt (private) -------------------------------------------------------
function TBruch.ggt(p1,p2:Integer) : Integer;
Var h : Integer;
begin
  p2 := Abs(p2); p1 := Abs(p1);
  if p2 > p1 then begin h:=p1; p1:=p2; p2:=h; end;
  if p2 > 0  then begin
                  while ((p1 mod p2)>0) do
                         begin
                         h  := p1;
                         p1 := p2;
                         p2 := h mod p2;
                         end;
                  result := p2;
                  end
             else result := p1;
end;

//-------- normiere (private) --------------------------------------------------
procedure TBruch.normiere;
begin
  if zNenner > 0 then exit;
  zNenner  := - zNenner;
  zZaehler := - zZaehler;
end;

//-------- kuerzeVoll (private) ------------------------------------------------
procedure TBruch.kuerze;
var ggt_:Integer;
begin
  ggt_ := ggt(zZaehler,zNenner);
  zNenner := zNenner  div ggt_;
  zZaehler:= zZaehler div ggt_;
end;

//-------- init (public) -------------------------------------------------------
constructor TBruch.init;
begin
  zZaehler := 1;
  zNenner  := 1;
end;

//-------- setzeBruch (public) -------------------------------------------------
procedure TBruch.setzeBruch(pZaehler,pNenner:Integer);
begin
  if pNenner = 0 then exit;
  zNenner  := pNenner;
  zZaehler := pZaehler;
  normiere;
  kuerze;
end;

//-------- setzeZaehler (public) -----------------------------------------------
procedure TBruch.setzeZaehler(pZaehler:Integer);
begin
  setzeBruch (pZaehler,zNenner);
end;

//-------- setzeNenner (public) ------------------------------------------------
procedure TBruch.setzeNenner(pNenner:Integer);
begin
  setzeBruch (zZaehler,pNenner);
end;

//-------- dezimalwert (public) ------------------------------------------------
function TBruch.dezimalwert : Real;
begin
  result := zZaehler/zNenner;
end;

//-------- nenner (public) -----------------------------------------------------
function TBruch.nenner : Integer;
begin
  result := zNenner;
end;

//-------- zaehler (public) ----------------------------------------------------
function TBruch.zaehler : Integer;
begin
  result := zZaehler;
end;

//-------- gibFrei (public) ----------------------------------------------------
destructor TBruch.gibFrei;
begin
  //nix!
end;

end.
