UNIT mTBruchrechner;
// nderungen:
// 1.
// Gegenber SuM-Version unverndert.
// 2.
// Alle Referenzen auf SuM entfernt
// => "ganzeZahl" -> "Integer"; "Zahl" -> "Real"
// Brche jetzt "private", Ebenso "macheGleichnamig"
// Die Methoden "SetzeBruchx" und die Anfrage "ergebnisbruch" sind berflssig,
// solange wie hier mit festen Argumentbrchen und festem Ergebnisbruch
// gearbeitet wird. Ihre Deklaration offenbart einen Denkfehler:
// "kenntBruchx" sind keine Kopien der Brche in "mFormBruchrechner", sondern
// Referenzen auf die dort erzeugten Brche. Damit ist jede dort an den Brchen
// erfolgte nderung auch fr den Rechenprozess verfgbar und umgekehrt steht
// das Rechenergebnis auch dort sofort zur Verfgung. Dies zeigt sich auch
// darin, dass diese Methoden in "mFormBruchrechner" nicht aufgerufen werden.
// 3.
// Die nderungen in "TBruch" wirken sich aus:
// Die Methode "setzeBruch" tritt fr das Ergebnis an die Stelle von
// "setzeNenner" bzw. "setzeZaehler" und das Krzen des Ergebnisses muss
// nicht mehr explizit aufgerufen werden.
// Neu ist, dass die Ausgangsbrche nicht mehr explizit verndert werden,
// sondern nur noch der Ergebnisbruch gesetzt wird. Dadurch werden die
// Prozeduren "erweitereUm" und "kuerzeUm" in TBruch berflssig.
// Vereinfachende Namensnderungen. Keine Absicherungen.
// 4.
// Der Bruchrechner wird "autonomer":
// Er kennt jetzt seine aktuelle Rechenart und akzeptiert nur noch die nderung
// dieser Rechenart und den Auftrag "rechne".
// Zu Ausgabezwecken kann das aktuelle Rechenzeichen zurckgeliefert werden.
// Keine Absicherungen.

interface

uses   mTBruch;
// Uses-Klausel ggf. anpassen

type
   TBruchrechner = CLASS

     private

     //Objektbeziehungen: kennt: kBruch1, kBruch2, kBruch3 : TBruch;

        kBruch1       : TBruch;
        kBruch2       : TBruch;
        kBruchE       : TBruch;

     // Attribute

        zVerknuepfung : Char;

     // Methoden

        function  kgv (a,b:Integer) : Integer;
        procedure addiere;
        procedure subtrahiere;
        procedure multipliziere;
        procedure dividiere;

     public

        constructor init(pBruch1,pBruch2,pBruchE:TBruch);
        procedure   setzeVerknuepfung  (pzeichen : Char);
        procedure   rechne;
        function    rechenzeichen : Char;
        destructor  gibFrei;

   end;

implementation

//-------- kgv (private) -------------------------------------------------------
function TBruchrechner.kgv (a,b:Integer): Integer;
Var h : Integer;
begin
  if a<b then begin h:=a; a:=b; b:=h; end;
  h := a;
  While (a mod b > 0) do a:= a+h;
  result := a;
end;
//-------- addiere (private) ---------------------------------------------------
procedure TBruchrechner.addiere;
Var hauptnenner : Integer;
begin
  hauptnenner := kgv(kBruch1.nenner,kBruch2.nenner);
  kBruchE.setzeBruch ( hauptnenner div kBruch1.nenner * kBruch1.zaehler
                     + hauptnenner div kBruch2.nenner * kBruch2.zaehler
                     , hauptnenner);
end;
//-------- subtrahiere (private) -----------------------------------------------
procedure TBruchrechner.subtrahiere;
Var hauptnenner : Integer;
begin
  hauptnenner := kgv(kBruch1.nenner,kBruch2.nenner);
  kBruchE.setzeBruch ( hauptnenner div kBruch1.nenner * kBruch1.zaehler
                     - hauptnenner div kBruch2.nenner * kBruch2.zaehler
                     , hauptnenner);
end;
//-------- multipliziere (private) ---------------------------------------------
procedure TBruchrechner.multipliziere;
begin
  kBruchE.setzeBruch ( kBruch1.zaehler * kBruch2.zaehler
                     , kBruch1.nenner  * kBruch2.nenner  );
end;
//-------- dividiere (private) -------------------------------------------------
procedure TBruchrechner.dividiere;
begin
  kBruchE.setzeBruch ( kBruch1.zaehler * kBruch2.nenner
                     , kBruch1.nenner  * kBruch2.zaehler );
end;

//-------- init (public) -------------------------------------------------------
constructor TBruchrechner.init(pBruch1,pBruch2,pbruchE:TBruch);
begin
  kBruch1       := pBruch1;
  kBruch2       := pBruch2;
  kBruchE       := pBruchE;
  zVerknuepfung := '+';
end;

//-------- setzeVerknuepfung (public) ------------------------------------------
procedure TBruchrechner.setzeVerknuepfung (pZeichen:Char);
begin
  zVerknuepfung := pZeichen;
end;

//-------- rechne (public) -----------------------------------------------------
procedure TBruchrechner.rechne;
begin
  case zVerknuepfung of
  '+' : addiere;
  '-' : subtrahiere;
  '*' : multipliziere;
  '/' : dividiere;
  end;
end;

//-------- rechenzeichen (public) ----------------------------------------------
function TBruchrechner.rechenzeichen : Char;
begin
  result := zVerknuepfung;
end;

//-------- gibFrei (public) ----------------------------------------------------
destructor TBruchrechner.gibFrei;
begin
  //nix!
end;

end.
