UNIT mTBruch;
// nderungen:
// 1.
// Function "ggT" benutzt Betrge, so dass auch Brche mit
// negativem Zhler oder Nenner richtig gekrzt werden.
// Hat eines der beiden Argumente den Wert 0, so ist das andere
// der ggT. Sonst gegenber SuM-Version unverndert.
// 2.
// Alle Referenzen auf SuM entfernt
// => "ganzeZahl" -> "Integer"; "Zahl" -> "Real"
// "zNenner", "zZaehler" sind private!
// 3.
// Die Unit "TBruch" produziert nun autonomere, "intelligentere" Brche:
// Die Setzung "Nenner = 0" wird ignoriert.
// Nach jeder nderung von Zhler oder Nenner wird der Bruch in die
// "Normalform" (Zhler aus Z, Nenner aus N) berfhrt ("normiere")
// und danach gekrzt. "kuerze(Voll)" kann dadurch "private" werden.
// Die Neucodierung von "TBruchrechner" ohne nderung der Ausgangsbrche
// macht die "technischen" Methoden "erweitereUm", "kuerzeUm" und "Kehrbruch"
// berflssig.
// Neu notwendig wird "setzeBruch", das nach einer kompletten nderung
// eines Bruches (ErgebnisBruch!) verhindert, dass Zhler oder Nenner,
// wer immer als erster Wert verndert wird, noch mit dem zu ber-
// schreibenden anderen Wert des alten Bruches verrechnet wird.
// 4.
// keine nderung
// 5.
// Das neue Attribut "zok" speichert, ob der letzte Import eines neuen Wertes
// zulssig war (also Nenner<>0 ).
// Die Anfrage "ok" macht die externe Auswertung mglich:
// Davon abhngig kann bei zulssigen nderungen die Rechnung ausgelst werden
// und bei einer unzulssigen Eingabe ggf. die Reaktion der Oberflche.
// Fr den zweiten Bruch reicht diese Prfung nicht aus,-
// im Falle der Division darf auch sein Zhler nicht den Wert null annehmen.
// Dafr gibt es die Tochterklasse "TBruchPlus",
// die ggf. den Zhler zustzlich prft.
// Dafr mssen das Attribut"zok" "protected" deklariert werden und
// die zu berschreibenden Methoden "virtual";
// 6.
// keine weitere nderung
// 7.
// "dezimalwert" verlagert nach "TBruchDarsteller",
// wird dort vorzeichenfrei implementiert

interface

// uses;

type
   TBruch = CLASS

     // Attribute

     protected
         zok : Boolean;

     private
         zNenner : Integer;
         zZaehler : Integer;

     // Methoden

     private

        procedure kuerze;
        procedure normiere;
        function ggt(p1,p2:Integer) : Integer;

     public

        constructor init; virtual;

        // Prozeduren fr den Import von Daten
        procedure setzeBruch(pZaehler,pNenner:Integer); virtual;
        procedure setzeNenner(pNenner:Integer);
        procedure setzeZaehler(pZaehler:Integer);

        // Prozeduren fr den Export von Daten
        function ok : Boolean;
        function nenner : Integer;
        function zaehler : Integer;

        destructor gibFrei;

   end;

implementation

//-------- kuerze (private) ----------------------------------------------------
procedure TBruch.kuerze;
var ggt_:Integer;
begin
  ggt_ := ggt(zZaehler,zNenner);
  zNenner := zNenner  div ggt_;
  zZaehler:= zZaehler div ggt_;
end;

//-------- normiere (private) --------------------------------------------------
procedure TBruch.normiere;
begin
  if zNenner > 0 then exit;
  zNenner  := - zNenner;
  zZaehler := - zZaehler;
end;

//-------- ggt (private) -------------------------------------------------------
function TBruch.ggt(p1,p2:Integer) : Integer;
Var h : Integer;
begin
  p2 := Abs(p2); p1 := Abs(p1);
  if p2 > p1 then begin h:=p1; p1:=p2; p2:=h; end;
  if p2 > 0  then begin
                  while ((p1 mod p2)>0) do
                         begin
                         h  := p1;
                         p1 := p2;
                         p2 := h mod p2;
                         end;
                  result := p2;
                  end
             else result := p1;
end;

//-------- init (public) -------------------------------------------------------
constructor TBruch.init;
begin
  zZaehler := 1;
  zNenner  := 1;
  zok      := true;
end;

//-------- setzeBruch (public) -------------------------------------------------
procedure TBruch.setzeBruch(pZaehler,pNenner:Integer);
begin
  if pNenner = 0
    then zok:=false
    else begin
         zok:=true;
         zNenner  := pNenner;
         zZaehler := pZaehler;
         normiere;
         kuerze;
         end;
end;

//-------- setzeNenner (public) ------------------------------------------------
procedure TBruch.setzeNenner(pNenner:Integer);
begin
  setzeBruch (zZaehler,pNenner);
end;

//-------- setzeZaehler (public) -----------------------------------------------
procedure TBruch.setzeZaehler(pZaehler:Integer);
begin
  setzeBruch (pZaehler,zNenner);
end;

//-------- ok (public) ---------------------------------------------------------
function TBruch.ok : Boolean;
begin
  result := zok;
end;

//-------- nenner (public) -----------------------------------------------------
function TBruch.nenner : Integer;
begin
  result := zNenner;
end;

//-------- zaehler (public) ----------------------------------------------------
function TBruch.zaehler : Integer;
begin
  result := zZaehler;
end;

//-------- gibFrei (public) ----------------------------------------------------
destructor TBruch.gibFrei;
begin
  //nix!
end;

end.
