UNIT mTBruchDarsteller;
// nderungen:
// 7.
// Die neue Klasse TBruchDarsteller bernimmt die Anzeige
// eines Bruches in drei Modi:
// - Darstellung als klassischer Bruch (ggf. unecht, Nenner=1, Zhler=0)
// - Darstellung als gemischter Bruch,
//   ggf. als Ganzzahl ohne Restbruch 0/x, ggf. als Bruch ohne Ganzzahl 0
// - Darstellung als Dezimalbruch im Format :6:3
// Dabei wird das Vorzeichen grundstzlich getrennt ausgegeben.
//
// Die Klasse verwaltet die Darstellungsmodi autonom in der Zustandsvariablen
// "zDarstellung" (benutzerdefinierter Aufzhlungstyp). Der Anwender kann
// die Darstellung mittels "setzeDarstellung" durch bergabe eines der drei
// Parameter "bruch", "gemischt" bzw. "dezimal" (oder 0,1,2) direkt umschalten.
// Alternativ knnen die drei Modi auch zyklisch durchgesteppt werden mittels
// "steppeDarstellung".
// Nach jeder nderung des Darstellungsmodus erfolgt eine Neudarstellung.
// Diese kann bei nderung anderer Parameter auch mittels "zeigeAn" von auen
// erzwungen werden.
//
// Die Darstellung erfolgt durch Umschalten der Sichtbarkeit von Etiketten, die
// auf der Oberflche passend erzeugt worden sein mssen und die bei der Initia-
// lisierung an das erzeugte Objekt vom Typ "TBruchDarsteller" bergeben werden.
// Soweit Werte ausgegeben werden, werden auch die Inhalte einzelner Etiketten
// verndert. Die Werte liefert "k(ennt)Bruch", diese Variable wird beim Initia-
// lisieren mit einem Verweis auf den darzustellenden Bruch belegt.
// Diese Vorgehensweise ermglicht es, mit mehreren Objekten dieser Klasse
// mehrere Brche an unterschiedlichen Orten des Fensters darzustellen.
//
// Intern benutzt "TBruchDarsteller" eigene Variablen fr Zhler (Absolutwert),
// Nenner und Vorzeichen des darzustellenden Bruches. Dies minimiert die Zahl
// der Zugriffe auf Dienste des darzustellenden Bruches. Die einzelnen Dar-
// stellungen sind private Methoden zugewiesen. Insgesamt werden berflssige
// Ausgabeoperationen mglichst vermieden.

interface

uses mEtikett, mTBruch;

type
   TDarstellung          = (bruch,gemischt,dezimal);
   TBruchDarsteller      = CLASS

     // Attribute

     private

         kBruch          :  TBruch;

         EAllVorzeichen  :  TEtikett;
         EStdBruchStrich :  TEtikett;
         EMixBruchStrich :  TEtikett;
         EMixGanzzahl    :  TEtikett;
         EMixZaehler     :  TEtikett;
         EMixNenner      :  TEtikett;
         EStdZaehler     :  TEtikett;
         EStdNenner      :  TEtikett;
         EDezimal        :  TEtikett;

         zDarstellung    :  TDarstellung;

         // interne Variablen ersparen stndige Funktionsaufrufe in "TBruch"
         zZaehler        :  Integer; // Betrag
         zNenner         :  Integer;
         zIstNegativ     :  Boolean;


     // Methoden

     private

       procedure setzeWerte;
       procedure zeigeAnVorzeichen;
       procedure zeigeAnBruch;
       procedure zeigeAnGemischt;
       procedure zeigeAnDezimal;

     public

        constructor  init
                     ( pBruch           :  TBruch;
                       pAllVorzeichen,
                       pStdBruchStrich,
                       pMixBruchStrich,
                       pMixGanzzahl,
                       pMixZaehler,
                       pMixNenner,
                       pStdZaehler,
                       pStdNenner,
                       pDezimal         :  TEtikett);

        // Prozeduren fr den Import von Daten
        procedure    setzeDarstellung
                     ( pDarstellung : TDarstellung);

        procedure    steppeDarstellung;

        // sonstige Auftrge
        procedure    zeigeAn;

        // Prozeduren zum Export von Daten
        // entfallen

        destructor   gibFrei;

   end;

implementation

//-------- init (public) -------------------------------------------------------
constructor TBruchDarsteller.init
            ( pBruch           :  TBruch;
              pAllVorzeichen,
              pStdBruchStrich,
              pMixBruchStrich,
              pMixGanzzahl,
              pMixZaehler,
              pMixNenner,
              pStdZaehler,
              pStdNenner,
              pDezimal         :  TEtikett);
begin
  kBruch          :=  pBruch;
  EAllVorzeichen  :=  pAllVorzeichen;
  EStdBruchStrich :=  pStdBruchStrich;
  EMixBruchStrich :=  pMixBruchStrich;
  EMixGanzzahl    :=  pMixGanzzahl;
  EMixZaehler     :=  pMixZaehler;
  EMixNenner      :=  pMixNenner;
  EStdZaehler     :=  pStdZaehler;
  EStdNenner      :=  pStdNenner;
  EDezimal        :=  pDezimal;

  zDarstellung    :=  bruch;
  self.zeigeAn;
end;

//-------- setzeDarstellung (public) -------------------------------------------
procedure TBruchDarsteller.setzeDarstellung (pDarstellung : TDarstellung);
begin
  zDarstellung := pDarstellung;

  case zDarstellung of
  bruch    :  zeigeAnBruch;             // self.zeigeAn wird nicht aufgerufen,
  gemischt :  zeigeAnGemischt;          // um unntiges neues Setzen der Daten
  dezimal  :  zeigeAnDezimal;           // zu vermeiden.
  end;
end;

//-------- steppeDarstellung (public) ------------------------------------------
procedure TBruchDarsteller.steppeDarstellung;
begin
  if zDarstellung < dezimal
    then inc(zDarstellung)
    else zDarstellung := bruch;

  case zDarstellung of
  bruch    :  zeigeAnBruch;             // self.zeigeAn wird nicht aufgerufen,
  gemischt :  zeigeAnGemischt;          // um unntiges neues Setzen der Daten
  dezimal  :  zeigeAnDezimal;           // zu vermeiden.
  end;
end;
//-------- zeigeAn (public) ----------------------------------------------------
procedure TBruchDarsteller.zeigeAn;
begin
  setzeWerte;
  zeigeAnVorzeichen;

  case zDarstellung of
  bruch    :  zeigeAnBruch;
  gemischt :  zeigeAnGemischt;
  dezimal  :  zeigeAnDezimal;
  end;
end;


//-------- setzeWerte (private) ------------------------------------------------
procedure TBruchDarsteller.setzeWerte;
begin
  zZaehler    := kBruch.zaehler;
  zNenner     := kBruch.nenner;
  zIstNegativ := zZaehler < 0;
  zZaehler    := Abs (zZaehler);
end;

//-------- zeigeAnVorzeichen (private) -----------------------------------------
procedure TBruchDarsteller.zeigeAnVorzeichen;
begin
  if zIstNegativ
    then EAllVorzeichen.zeige       // Inhalt in mFormBruchrechner festgelegt!
    else EAllVorzeichen.verstecke;
end;

//-------- zeigeAnBruch (private) ----------------------------------------------
procedure TBruchDarsteller.zeigeAnBruch;
begin
  EMixBruchStrich.verstecke;            // alle anderen Etiketten verschwinden
  EMixGanzzahl.verstecke;
  EMixZaehler.verstecke;
  EMixNenner.verstecke;
  EDezimal.verstecke;

  EStdBruchStrich.zeige;                // Anzeige erstellen
  EStdZaehler.setzeInhalt(zZaehler);
  EStdNenner.setzeInhalt(zNenner);
  EStdZaehler.zeige;
  EStdNenner.zeige;
end;

//-------- zeigeAnGemischt (private) -------------------------------------------
procedure TBruchDarsteller.zeigeAnGemischt;
var ganzzahl : Integer;
begin

  EStdBruchStrich.verstecke;            // alle anderen Etiketten verschwinden
  EStdZaehler.verstecke;
  EStdNenner.verstecke;
  EDezimal.verstecke;

  ganzzahl := zZaehler div zNenner;
  if (zZaehler mod zNenner) > 0         // Restbruch vorhanden?
    then if ganzzahl > 0
           then begin
                EMixGanzzahl.setzeInhalt(ganzzahl);
                EMixGanzzahl.zeige;
                EMixZaehler.setzeInhalt (zZaehler mod zNenner);
                EMixNenner.setzeInhalt  (zNenner);
                EMixBruchStrich.zeige;
                EMixZaehler.zeige;
                EMixNenner.zeige;
                end
           else zeigeAnBruch
    else begin
         EMixZaehler.verstecke;
         EMixNenner.verstecke;
         EMixBruchStrich.verstecke;
         EMixGanzzahl.setzeInhalt(ganzzahl);
         EMixGanzzahl.zeige;
         end;
end;

//-------- zeigeAnDezimal (private) --------------------------------------------
procedure TBruchDarsteller.zeigeAnDezimal;
var       ausStr  : String[6];
begin
  EStdBruchStrich.verstecke;            // alle anderen Etiketten verschwinden
  EStdZaehler.verstecke;
  EStdNenner.verstecke;
  EMixBruchStrich.verstecke;
  EMixGanzzahl.verstecke;
  EMixZaehler.verstecke;
  EMixNenner.verstecke;

  Str((zZaehler/zNenner):6:3,ausStr);
  EDezimal.setzeInhalt(ausStr);
  EDezimal.zeige;

end;


//-------- gibFrei (public) ----------------------------------------------------
destructor TBruchDarsteller.gibFrei;
begin
  //nix!
end;

end.
