UNIT mTBruchDarsteller;
// nderungen:
// 7.
// Die neue Klasse TBruchDarsteller bernimmt die Anzeige
// eines Bruches in drei Modi:
// - Darstellung als klassischer Bruch (ggf. unecht, Nenner=1, Zhler=0)
// - Darstellung als gemischter Bruch,
//   ggf. als Ganzzahl ohne Restbruch 0/x, ggf. als Bruch ohne Ganzzahl 0
// - Darstellung als Dezimalbruch im Format :6:3
// Dabei wird das Vorzeichen grundstzlich getrennt ausgegeben.
//
// Die Klasse verwaltet die Darstellungsmodi autonom in der Zustandsvariablen
// "zDarstellung" (benutzerdefinierter Aufzhlungstyp). Der Anwender kann
// die Darstellung mittels "setzeDarstellung" durch bergabe eines der drei
// Parameter "bruch", "gemischt" bzw. "dezimal" (oder 0,1,2) direkt umschalten.
// Alternativ knnen die drei Modi auch zyklisch durchgesteppt werden mittels
// "steppeDarstellung".
// Nach jeder nderung des Darstellungsmodus erfolgt eine Neudarstellung.
// Diese kann bei nderung anderer Parameter auch mittels "zeigeAn" von auen
// erzwungen werden.
//
// Die Darstellung erfolgt durch Umschalten der Sichtbarkeit von Etiketten, die
// auf der Oberflche passend erzeugt worden sein mssen und die bei der Initia-
// lisierung an das erzeugte Objekt vom Typ "TBruchDarsteller" bergeben werden.
// Soweit Werte ausgegeben werden, werden auch die Inhalte einzelner Etiketten
// verndert. Die Werte liefert "k(ennt)Bruch", diese Variable wird beim Initia-
// lisieren mit einem Verweis auf den darzustellenden Bruch belegt.
// Diese Vorgehensweise ermglicht es, mit mehreren Objekten dieser Klasse
// mehrere Brche an unterschiedlichen Orten des Fensters darzustellen.
//
// Intern benutzt "TBruchDarsteller" eigene Variablen fr Zhler (Absolutwert),
// Nenner und Vorzeichen des darzustellenden Bruches. Dies minimiert die Zahl
// der Zugriffe auf Dienste des darzustellenden Bruches. Den einzelnen Dar-
// stellungen sind private Methoden zugewiesen. Insgesamt werden berflssige
// Ausgabeoperationen mglichst vermieden.
// 8.
// Die Klasse Bruchdarsteller bekommt zwecks Anzeige der Ausgangsbrche in
// "mFormBruchRechner" zustzlich die Mglichkeit, die Darstellung ggf. 
// auch zu verbergen.
//
// Dazu dienen die Statusvariable "zSichtbar" und die neue Anfrage "verstecke".
// Aus Kompatibilittsgrnden mit den Oberflchenobjekten heit "zeigeAn"
// jetzt "zeige".
//
// Der Anfangszustand bei der Initialisierung ist zunchst "nicht sichtbar",
// dem entsprechend erfolgt ohne den Aufruf von "zeige" auch keine Anzeige.
// Des einfachen Gebrauchs wegen ndern die Methoden zum Setzen des Anzeige-
// modus auch die Ausgabe, sofern "zSichtbar" gesetzt ist. Sonst wird nur der
// Wert der Statusvariablen gendert.
//
// Innerhalb von Termen muss der darzustellende Bruch in Klammern eingeschlossen
// werden, falls er negativ ist. Deshalb erhlt die Methode "init" zwei weitere
// Parameter "pKlammerLinks" und "pKlammerRechts" vom Typ "TEtikett". Sofern
// hier Parameter bergeben werden, die von NIL verschieden sind, wird die
// neue Zustandsvariable "zMitKlammern" gesetzt, die dafr sorgt, dass ggf.
// die entsprechenden Klammern zusammen mit dem Vorzeichen ausgegeben werden.
// Wird "NIL" bergeben, fehlen die Klammern und es erfolgt die bliche Dar-
// stellung.
//
// Dies ginge mit Java viel einfacher: Man knnte einfach einen zweiten Kon-
// struktor mit abweichender Parameterliste schreiben, der den ersten berldt.
// Hier knnte man allenfalls eine Tochterklasse "TBruchDarstellerPlus" o..
// schreiben, die per "overload" dann auch einen Konstruktor mit abweichender
// Parameterliste besitzen kann.

interface

uses mEtikett, mTBruch;

type
   TDarstellung          = (bruch,gemischt,dezimal);
   TBruchDarsteller      = CLASS

     // Attribute

     private

         kBruch          :  TBruch;

         EtikettAllVorzeichen  :  TEtikett;
         EtikettStdBruchStrich :  TEtikett;
         EtikettMixBruchStrich :  TEtikett;
         EtikettMixGanzzahl    :  TEtikett;
         EtikettMixZaehler     :  TEtikett;
         EtikettMixNenner      :  TEtikett;
         EtikettStdZaehler     :  TEtikett;
         EtikettStdNenner      :  TEtikett;
         EtikettDezimal        :  TEtikett;

         EtikettKlammerLinks   :  TEtikett;
         EtikettKlammerRechts  :  TEtikett;

         zDarstellung    :  TDarstellung;
         zSichtbar       :  Boolean;
         zMitKlammern    :  Boolean;

         // interne Variablen ersparen stndige Funktionsaufrufe in "TBruch"

         zZaehler        :  Integer; // Betrag
         zNenner         :  Integer;
         zIstNegativ     :  Boolean;


     // Methoden

     private

       procedure setzeWerte;
       procedure zeigeAnVorzeichen;
       procedure zeigeAnWert;
       procedure zeigeAnBruch;
       procedure zeigeAnGemischt;
       procedure zeigeAnDezimal;

     public

        constructor  init
                     ( pBruch           :  TBruch;
                       pAllVorzeichen,
                       pStdBruchStrich,
                       pMixBruchStrich,
                       pMixGanzzahl,
                       pMixZaehler,
                       pMixNenner,
                       pStdZaehler,
                       pStdNenner,
                       pDezimal,
                       pKlammerLinks,
                       pKlammerRechts   :  TEtikett );

        // Prozeduren fr den Import von Daten
        procedure    setzeDarstellung
                     ( pDarstellung : TDarstellung);

        procedure    steppeDarstellung;

        // sonstige Auftrge
        procedure    zeige;
        procedure    verstecke;

        // Prozeduren zum Export von Daten
        function     modus : TDarstellung;

        destructor   gibFrei;

   end;

implementation

//-------- init (public) -------------------------------------------------------
constructor TBruchDarsteller.init
            ( pBruch           :  TBruch;
              pAllVorzeichen,
              pStdBruchStrich,
              pMixBruchStrich,
              pMixGanzzahl,
              pMixZaehler,
              pMixNenner,
              pStdZaehler,
              pStdNenner,
              pDezimal,
              pKlammerLinks,
              pKlammerRechts   :  TEtikett );

begin

  kBruch          :=  pBruch;

  EtikettAllVorzeichen  :=  pAllVorzeichen;
  EtikettStdBruchStrich :=  pStdBruchStrich;
  EtikettMixBruchStrich :=  pMixBruchStrich;
  EtikettMixGanzzahl    :=  pMixGanzzahl;
  EtikettMixZaehler     :=  pMixZaehler;
  EtikettMixNenner      :=  pMixNenner;
  EtikettStdZaehler     :=  pStdZaehler;
  EtikettStdNenner      :=  pStdNenner;
  EtikettDezimal        :=  pDezimal;
  EtikettKlammerLinks   :=  pKlammerLinks;
  EtikettKlammerRechts  :=  pKlammerRechts;

  zMitKlammern    :=  (pKlammerLinks <> NIL) AND (pKlammerRechts <> NIL);
  zDarstellung    :=  bruch;
  zSichtbar       :=  false;

end;

//-------- setzeDarstellung (public) -------------------------------------------
procedure TBruchDarsteller.setzeDarstellung (pDarstellung : TDarstellung);
begin
  zDarstellung := pDarstellung;
  if zSichtbar then zeigeAnWert;
end;

//-------- steppeDarstellung (public) ------------------------------------------
procedure TBruchDarsteller.steppeDarstellung;
begin
  if zDarstellung < dezimal
    then inc(zDarstellung)
    else zDarstellung := bruch;
  if zSichtbar then zeigeAnWert;
end;

//-------- zeige (public) ------------------------------------------------------
procedure TBruchDarsteller.zeige;
begin
  zSichtbar := true;
  setzeWerte;
  zeigeAnVorzeichen;
  zeigeAnWert;
end;

//-------- modus (public) ------------------------------------------------------
function TbruchDarsteller.modus : TDarstellung;
begin
result := zDarstellung;
end;

// ------- verstecke (public) --------------------------------------------------
procedure TBruchDarsteller.verstecke;
begin
  zSichtbar := false;
  EtikettAllVorzeichen.verstecke;
  EtikettStdBruchStrich.verstecke;
  EtikettStdZaehler.verstecke;
  EtikettStdNenner.verstecke;
  EtikettMixBruchStrich.verstecke;
  EtikettMixGanzzahl.verstecke;
  EtikettMixZaehler.verstecke;
  EtikettMixNenner.verstecke;
  EtikettDezimal.verstecke;
  if not zMitKlammern then exit;
  EtikettKlammerLinks.verstecke;
  EtikettKlammerRechts.verstecke;
end;

//-------- gibFrei (public) ----------------------------------------------------
destructor TBruchDarsteller.gibFrei;
begin
  //nix!
end;


//-------- setzeWerte (private) ------------------------------------------------
procedure TBruchDarsteller.setzeWerte;
begin
  zZaehler    := kBruch.zaehler;
  zNenner     := kBruch.nenner;
  zIstNegativ := zZaehler < 0;
  zZaehler    := Abs (zZaehler);
end;

//-------- zeigeAnVorzeichen (private) -----------------------------------------
procedure TBruchDarsteller.zeigeAnVorzeichen;
begin
  if zIstNegativ
    then begin
         EtikettAllVorzeichen.zeige;
         if not zMitKlammern then exit;
         EtikettKlammerLinks.zeige;
         EtikettKlammerRechts.zeige;
         end
    else begin
         EtikettAllVorzeichen.verstecke;
         if not zMitKlammern then exit;
         EtikettKlammerLinks.verstecke;
         EtikettKlammerRechts.verstecke;
         end;
end;

//-------- zeigeAnWert (private) -----------------------------------------------
procedure TBruchDarsteller.zeigeAnWert;
begin
  case zDarstellung of
  bruch    :  zeigeAnBruch;
  gemischt :  zeigeAnGemischt;
  dezimal  :  zeigeAnDezimal;
  end;
end;

//-------- zeigeAnBruch (private) ----------------------------------------------
procedure TBruchDarsteller.zeigeAnBruch;
begin
  EtikettMixBruchStrich.verstecke;      // alle anderen Etiketten verschwinden
  EtikettMixGanzzahl.verstecke;
  EtikettMixZaehler.verstecke;
  EtikettMixNenner.verstecke;
  EtikettDezimal.verstecke;

  EtikettStdBruchStrich.zeige;          // Anzeige erstellen
  EtikettStdZaehler.setzeInhalt(zZaehler);
  EtikettStdNenner.setzeInhalt(zNenner);
  EtikettStdZaehler.zeige;
  EtikettStdNenner.zeige;
end;

//-------- zeigeAnGemischt (private) -------------------------------------------
procedure TBruchDarsteller.zeigeAnGemischt;
var ganzzahl : Integer;
begin

  EtikettStdBruchStrich.verstecke;      // alle anderen Etiketten verschwinden
  EtikettStdZaehler.verstecke;
  EtikettStdNenner.verstecke;
  EtikettDezimal.verstecke;

  ganzzahl := zZaehler div zNenner;
  if (zZaehler mod zNenner) > 0         // Restbruch vorhanden?
    then if ganzzahl > 0
           then begin
                EtikettMixGanzzahl.setzeInhalt(ganzzahl);
                EtikettMixGanzzahl.zeige;
                EtikettMixZaehler.setzeInhalt (zZaehler mod zNenner);
                EtikettMixNenner.setzeInhalt  (zNenner);
                EtikettMixBruchStrich.zeige;
                EtikettMixZaehler.zeige;
                EtikettMixNenner.zeige;
                end
           else zeigeAnBruch
    else begin
         EtikettMixZaehler.verstecke;
         EtikettMixNenner.verstecke;
         EtikettMixBruchStrich.verstecke;
         EtikettMixGanzzahl.setzeInhalt(ganzzahl);
         EtikettMixGanzzahl.zeige;
         end;
end;

//-------- zeigeAnDezimal (private) --------------------------------------------
procedure TBruchDarsteller.zeigeAnDezimal;
var       ausStr  : String[6];
begin
  EtikettStdBruchStrich.verstecke;      // alle anderen Etiketten verschwinden
  EtikettStdZaehler.verstecke;
  EtikettStdNenner.verstecke;
  EtikettMixBruchStrich.verstecke;
  EtikettMixGanzzahl.verstecke;
  EtikettMixZaehler.verstecke;
  EtikettMixNenner.verstecke;

  Str((zZaehler/zNenner):6:3,ausStr);
  EtikettDezimal.setzeInhalt(ausStr);
  EtikettDezimal.zeige;

end;

end.
